<?php
/**
 *
 * Licentia, Unipessoal LDA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://licentia.pt/magento-license.txt
 *
 * @title      Licentia Panda - Magento® Sales Automation Extension
 * @package    Licentia
 * @author     Bento Vilas Boas <bento@licentia.pt>
 * @copyright  Copyright (c) 2012-2017 Licentia - https://licentia.pt
 * @license    https://licentia.pt/magento-license.txt
 *
 */

namespace Egoi\Marketing\Observer;

use Magento\Framework\Event\ObserverInterface;

/**
 * Class ConversionInit
 *
 * @package Egoi\Marketing\Observer
 */
class ConversionInit implements ObserverInterface
{

    /**
     * @var \Magento\Framework\Stdlib\CookieManagerInterface
     */
    protected $cookieManager;

    /**
     * @var \Magento\Framework\Stdlib\Cookie\PublicCookieMetadata
     */
    protected $cookieMetadataFactory;

    /**
     * ConversionInit constructor.
     *
     * @param \Magento\Framework\Stdlib\CookieManagerInterface      $cookieManager
     * @param \Magento\Framework\Stdlib\Cookie\PublicCookieMetadata $publicCookieMetadata
     */
    public function __construct(
        \Magento\Framework\Stdlib\CookieManagerInterface $cookieManager,
        \Magento\Framework\Stdlib\Cookie\PublicCookieMetadata $publicCookieMetadata)
    {


        $this->cookieManager = $cookieManager;
        $this->cookieMetadataFactory = $publicCookieMetadata;
    }

    /**
     * @param \Magento\Framework\Event\Observer $event
     */
    public function execute(\Magento\Framework\Event\Observer $event)
    {

        try {
            /** @var \Magento\Framework\App\RequestInterface $request */
            $request = $event->getEvent()->getRequest();

            if ($request->getParam('utm_source') == 'e-goi') {
                $metadata = $this->cookieMetadataFactory->setDuration(3600 * 24 * 3)
                                                        ->setPath('/');

                $this->cookieManager->setPublicCookie(
                    \Egoi\Marketing\Model\Conversions::COOKIE_NAME,
                    json_encode($request->getParams()),
                    $metadata
                );
            }

        } catch (\Exception $e) {

        }
    }

}
